// SerAddressBookMDIDoc.cpp : implementation of the CSerAddressBookMDIDoc class
//

#include "stdafx.h"
#include "SerAddressBookMDI.h"
#include "SerAddressBookMDIDoc.h"
#include <string>
#include "Contact.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CSerAddressBookMDIDoc

IMPLEMENT_DYNCREATE(CSerAddressBookMDIDoc, CDocument)

BEGIN_MESSAGE_MAP(CSerAddressBookMDIDoc, CDocument)
END_MESSAGE_MAP()


// CSerAddressBookMDIDoc construction/destruction

CSerAddressBookMDIDoc::CSerAddressBookMDIDoc()
		: m_nFileVersionSchema(0)
{
	// TODO: add one-time construction code here

}

CSerAddressBookMDIDoc::~CSerAddressBookMDIDoc()
{
}

BOOL CSerAddressBookMDIDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)

	return TRUE;
}

// CSerAddressBookMDIDoc serialization
void CSerAddressBookMDIDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())			// storing
	{
		ar << m_nFileVersionSchema;

		// write the number of contacts
		ar << (int)m_cContactsList.GetCount();
		Contact::SetCurrentContactVersion(m_nFileVersionSchema);

		// write all the contacts
		POSITION pos = m_cContactsList.GetHeadPosition();
		while(pos != NULL)
		{
			Contact contact = m_cContactsList.GetNext(pos);

			contact.Serialize(ar);
		}
	}
	else						// loading
	{
		ar >> m_nFileVersionSchema;

		m_cContactsList.RemoveAll();

		int count = 0;
		ar >> count;

		// read the number of contacts
		while (count-- > 0)
		{
			Contact contact;
			contact.Serialize(ar);

			m_cContactsList.AddTail(contact);
		}
	}

	UpdateAllViews(NULL);
}

// CSerAddressBookMDIDoc diagnostics

#ifdef _DEBUG
void CSerAddressBookMDIDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CSerAddressBookMDIDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

bool CSerAddressBookMDIDoc::AddContact(const Contact& contact)
{
	if(FindContact(contact.GetFirstName()) == NULL)
	{
		m_cContactsList.AddTail(contact);
		return true;
	}

	return false;
}

bool CSerAddressBookMDIDoc::RemoveContact(const CString& firstname, const CString& lastname)
{
	POSITION pos = m_cContactsList.GetHeadPosition();
	while(pos != NULL)
	{
		POSITION former = pos;
		Contact contact = m_cContactsList.GetNext(pos);
		if(contact.GetFirstName().Compare(firstname) == 0 &&
			contact.GetLastName().Compare(lastname) == 0)
		{
			m_cContactsList.RemoveAt(former);
			return true;
		}
	}

	return false;
}

POSITION CSerAddressBookMDIDoc::FindContact(const CString& firstname) const
{
	POSITION pos = m_cContactsList.GetHeadPosition();
	while(pos != NULL)
	{
		POSITION former = pos;
		Contact contact = m_cContactsList.GetNext(pos);

		if(contact.GetFirstName().Compare(firstname) == 0)
			return former;
	}

	return NULL;
}

bool CSerAddressBookMDIDoc::FindContact(const CString& firstname, Contact& contact) const
{
	POSITION pos = m_cContactsList.GetHeadPosition();
	while(pos != NULL)
	{
		contact = m_cContactsList.GetNext(pos);
		if(contact.GetFirstName().Compare(firstname) == 0)
		{
			return true;
		}
	}

	return false;
}

bool CSerAddressBookMDIDoc::UpdateContact(const CString& firstname, Contact& contact) 
{
	if (FindContact(firstname) != NULL)
	{
		POSITION current_pos;
		current_pos = FindContact(firstname);
	
		Contact con;
		con = m_cContactsList.GetAt(current_pos);
		con = contact;
		m_cContactsList.SetAt(current_pos, con);

		return true;
	}

	return false;
}

BOOL CSerAddressBookMDIDoc::DoSave(LPCTSTR lpszPathName, BOOL bReplace)
{
	CString newName = lpszPathName;
	if (newName.IsEmpty())
	{
		CDocTemplate* pTemplate = GetDocTemplate();
		ASSERT(pTemplate != NULL);

		newName = m_strPathName;
		if (bReplace && newName.IsEmpty())
		{
			newName = m_strTitle;
			// check for dubious filename
			int iBad = newName.FindOneOf(_T(" #%;/\\"));
			if (iBad != -1)
				newName.ReleaseBuffer(iBad);
	}

		if (!AfxGetApp()->DoPromptFileName(newName,
					bReplace ? AFX_IDS_SAVEFILE : AFX_IDS_SAVEFILECOPY,
					OFN_HIDEREADONLY | OFN_PATHMUSTEXIST, FALSE, NULL))
		return FALSE;       // don't even attempt to save
	}

	CWaitCursor wait;

	if (!OnSaveDocument(newName))
	{
		if (lpszPathName == NULL)
		{
			// be sure to delete the file
			TRY
			{
				CFile::Remove(newName);
			}
			CATCH_ALL(e)
			{
				TRACE0("Warning: failed to delete file after failed SaveAs.\n");
				e->Delete();     //modified
			}
			END_CATCH_ALL
		}
		return FALSE;
	}

	// reset the title and change the document name
	if (bReplace)
		SetPathName(newName);

	return TRUE;        // success
}  

BOOL CSerAddressBookMDIDoc::OnSaveDocument(LPCTSTR lpszPathName)
{

	std::wstring file_name = lpszPathName;
	if(file_name.substr(file_name.find_last_of(L".") + 1) == L"sab1") {
		m_nFileVersionSchema = 1;
		Contact::SetCurrentContactVersion(1);
	} 
	else 
	if(file_name.substr(file_name.find_last_of(L".") + 1) == L"sab2") {
		m_nFileVersionSchema = 2;
		Contact::SetCurrentContactVersion(2);
	}
	else
	{
		m_nFileVersionSchema = 2;
		Contact::SetCurrentContactVersion(m_nFileVersionSchema);
	}

	return CDocument::OnSaveDocument(lpszPathName);
}
